﻿Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles

<Serializable()>
Public Class CustomAreaStyle
	Inherits AreaStyle
	Public Property OutlineStyle As LineStyle = Nothing
	Public Property UseCustomOutline As Boolean = False
	Private moNullPen As New GeoPen(GeoColor.SimpleColors.Transparent, 0)
	Public Sub New()
		MyBase.New()
	End Sub
	Public Sub New(oBrush As GeoSolidBrush)
		MyBase.New()
		'***** This constructor is primarily for use with the page layout system
		Me.FillSolidBrush = oBrush

		Dim oInnerPen As New GeoPen(GeoColor.SimpleColors.White, 1)
		Dim oCenterPen As New GeoPen(GeoColor.SimpleColors.Transparent, 0)
		Dim oOuterPen As New GeoPen(GeoColor.StandardColors.Gray, 3)

		oOuterPen.DashStyle = LineDashStyle.Dot
		OutlineStyle = New LineStyle(oOuterPen, oInnerPen, oCenterPen)

		OutlinePen = New GeoPen(GeoColor.SimpleColors.Transparent, 0)
		PenBrushDrawingOrder = PenBrushDrawingOrder.PenFirst
		'PenBrushDrawingOrder = ThinkGeo.MapSuite.Core.PenBrushDrawingOrder.BrushFirst
		UseCustomOutline = True
	End Sub
	Public Sub New(oPen As GeoPen, oSolidBrush As GeoSolidBrush)
		MyBase.New(oPen, oSolidBrush)
		UseCustomOutline = False
		PenBrushDrawingOrder = PenBrushDrawingOrder.BrushFirst
	End Sub
	Public Sub New(oLineStyle As LineStyle, oSolidBrush As GeoSolidBrush)
		OutlinePen = moNullPen
		FillSolidBrush = oSolidBrush
		OutlineStyle = oLineStyle
		UseCustomOutline = True
		PenBrushDrawingOrder = PenBrushDrawingOrder.BrushFirst
	End Sub
	Protected Overrides Sub DrawCore(features As System.Collections.Generic.IEnumerable(Of Feature), canvas As GeoCanvas, labelsInThisLayer As System.Collections.ObjectModel.Collection(Of SimpleCandidate), labelsInAllLayers As System.Collections.ObjectModel.Collection(Of SimpleCandidate))
		Dim oLinesToDraw As Collection(Of LineShape)
		Dim oGdiPlusCanvas As PlatformGeoCanvas = TryCast(canvas, PlatformGeoCanvas)

		If Not oGdiPlusCanvas Is Nothing Then
			oGdiPlusCanvas.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAlias
			oGdiPlusCanvas.SmoothingMode = Drawing.Drawing2D.SmoothingMode.HighSpeed
		End If

		If UseCustomOutline Then
			MyBase.OutlinePen = moNullPen
			'MyBase.PenBrushDrawingOrder = ThinkGeo.MapSuite.Core.PenBrushDrawingOrder.PenFirst
			MyBase.DrawCore(features, canvas, labelsInThisLayer, labelsInAllLayers)
			For Each oFeature In features
				'***** Each of these will be a polygonshape or MultiPolygonShape
				'***** and each needs to be converted into a collection of lines
				'***** before our custom outline style can be drawn
				oLinesToDraw = PolygonToLinesCollection(oFeature.GetShape)
				OutlineStyle.Draw(oLinesToDraw, canvas, labelsInAllLayers, labelsInAllLayers)
			Next oFeature
		Else
			MyBase.DrawCore(features, canvas, labelsInThisLayer, labelsInAllLayers)
		End If

	End Sub
	Public Sub New(oAreaStyle As AreaStyle)
		'***** Create a new custom area style from an ordinary one
		With oAreaStyle
			Advanced.FillCustomBrush = .Advanced.FillCustomBrush
			If .CustomAreaStyles IsNot Nothing Then
				For Each oStyle In .CustomAreaStyles
					CustomAreaStyles.Add(oStyle)
				Next oStyle
			End If
			Me.DrawingLevel = .DrawingLevel
			Me.FillSolidBrush = .FillSolidBrush
			'***** Was getting an occasional error here because oAreaStyle.Filters
			'***** evaluated to Nothing.
			If .Filters IsNot Nothing Then
				For Each sString As String In .Filters
					Me.Filters.Add(sString)
				Next sString
			End If
			Me.IsActive = .IsActive
			Me.Name = .Name
			Me.OutlinePen = .OutlinePen
			Me.PenBrushDrawingOrder = .PenBrushDrawingOrder
			If .RequiredColumnNames IsNot Nothing Then
				For Each sString In .RequiredColumnNames
					Me.RequiredColumnNames.Add(sString)
				Next sString
			End If
			Me.XOffsetInPixel = .XOffsetInPixel
			Me.YOffsetInPixel = .YOffsetInPixel
			UseCustomOutline = False
			OutlineStyle = Nothing
		End With
	End Sub
	Public Shared Function PolygonToLinesCollection(oPolygon As BaseShape) As Collection(Of LineShape)
		Dim oLines As New Collection(Of LineShape)

		If TypeOf oPolygon Is PolygonShape Then
			oLines.Add(New LineShape(CType(oPolygon, PolygonShape).OuterRing.Vertices))
			For Each oRing As RingShape In CType(oPolygon, PolygonShape).InnerRings
				oLines.Add(New LineShape(oRing.Vertices))
			Next oRing
		ElseIf TypeOf oPolygon Is MultipolygonShape Then
			For Each oPoly As PolygonShape In CType(oPolygon, MultipolygonShape).Polygons
				oLines.Add(New LineShape(oPoly.OuterRing.Vertices))
				For Each oRing As RingShape In oPoly.InnerRings
					oLines.Add(New LineShape(oRing.Vertices))
				Next oRing
			Next oPoly
		End If
		Return oLines
	End Function

End Class
